@@ -20,6 +20,7 @@
 #include <cstdint>
 #include <queue>
 #include <vector>
+#include <xmmintrin.h>
 
 #include "./libdivide.h"
 
@@ -136,6 +137,17 @@ struct HeapNodeCompare {
   }
 };
 
+#define DIJKSTRA_3D_PREFETCH_26WAY(field) \
+  _mm_prefetch(reinterpret_cast<char*>(&field[loc - 1]), _MM_HINT_T0); \
+  _mm_prefetch(reinterpret_cast<char*>(&field[loc + sxy - 1]), _MM_HINT_T0); \
+  _mm_prefetch(reinterpret_cast<char*>(&field[loc - sxy - 1]), _MM_HINT_T0); \
+  _mm_prefetch(reinterpret_cast<char*>(&field[loc + sxy + sx - 1]), _MM_HINT_T0); \
+  _mm_prefetch(reinterpret_cast<char*>(&field[loc + sxy - sx - 1]), _MM_HINT_T0); \
+  _mm_prefetch(reinterpret_cast<char*>(&field[loc - sxy + sx - 1]), _MM_HINT_T0); \
+  _mm_prefetch(reinterpret_cast<char*>(&field[loc - sxy - sx - 1]), _MM_HINT_T0); \
+  _mm_prefetch(reinterpret_cast<char*>(&field[loc + sx - 1]), _MM_HINT_T0); \
+  _mm_prefetch(reinterpret_cast<char*>(&field[loc - sx - 1]), _MM_HINT_T0);
+
 /* Perform dijkstra's shortest path algorithm
  * on a 3D image grid. Vertices are voxels and
  * edges are the 26 nearest neighbors (except
@@ -208,6 +220,12 @@ std::vector<OUT> dijkstra3d(
       continue;
     }
 
+    // As early as possible, start fetching the
+    // data from RAM b/c the annotated lines below
+    // have 30-50% cache miss.
+    DIJKSTRA_3D_PREFETCH_26WAY(field)
+    DIJKSTRA_3D_PREFETCH_26WAY(dist)
+    
     if (power_of_two) {
       z = loc >> (xshift + yshift);
       y = (loc - (z << (xshift + yshift))) >> xshift;
@@ -227,12 +245,12 @@ std::vector<OUT> dijkstra3d(
       }
 
       neighboridx = loc + neighborhood[i];
-      delta = static_cast<float>(field[neighboridx]);
+      delta = static_cast<float>(field[neighboridx]); // high cache miss
 
       // Visited nodes are negative and thus the current node
       // will always be less than as field is filled with non-negative
       // integers.
-      if (dist[loc] + delta < dist[neighboridx]) { 
+      if (dist[loc] + delta < dist[neighboridx]) { // high cache miss
         dist[neighboridx] = dist[loc] + delta;
         parents[neighboridx] = loc + 1; // +1 to avoid 0 ambiguity
 
@@ -383,6 +401,15 @@ std::vector<OUT> bidirectional_dijkstra3d(
       }
     }
 
+    DIJKSTRA_3D_PREFETCH_26WAY(field)
+
+    if (forward) {
+      DIJKSTRA_3D_PREFETCH_26WAY(dist_fwd)
+    }
+    else {
+      DIJKSTRA_3D_PREFETCH_26WAY(dist_rev)
+    }
+
     if (power_of_two) {
       z = loc >> (xshift + yshift);
       y = (loc - (z << (xshift + yshift))) >> xshift;
@@ -514,6 +541,9 @@ std::vector<OUT> compass_guided_dijkstra3d(
       continue;
     }
 
+    DIJKSTRA_3D_PREFETCH_26WAY(field)
+    DIJKSTRA_3D_PREFETCH_26WAY(dist)
+
     xyzfn(loc);
 
     compute_neighborhood(neighborhood, x, y, z, sx, sy, sz);
@@ -649,6 +679,9 @@ OUT* parental_field3d(
       continue;
     }
 
+    DIJKSTRA_3D_PREFETCH_26WAY(field)
+    DIJKSTRA_3D_PREFETCH_26WAY(dist)
+
     if (power_of_two) {
       z = loc >> (xshift + yshift);
       y = (loc - (z << (xshift + yshift))) >> xshift;
@@ -958,6 +991,12 @@ float* euclidean_distance_field3d(
       continue;
     }
 
+    // As early as possible, start fetching the
+    // data from RAM b/c the annotated lines below
+    // have 30-50% cache miss.
+    DIJKSTRA_3D_PREFETCH_26WAY(field)
+    DIJKSTRA_3D_PREFETCH_26WAY(dist)
+
     if (power_of_two) {
       z = loc >> (xshift + yshift);
       y = (loc - (z << (xshift + yshift))) >> xshift;
@@ -1002,6 +1041,8 @@ float* euclidean_distance_field3d(
   return dist;
 }
 
+#undef DIJKSTRA_3D_PREFETCH_26WAY
+
 }; // namespace dijkstra3d
 
 #endif
\ No newline at end of file
